// vcc: 5V
// 
// K型熱電対温度センサモジュールキット(SPI接続)MAX31855使用(5V版/3.3V版)サンプルスケッチ
// Example sketch for MAX31855 Type-K thermocouple sensor module (5V/3.3V) 
// Switch-Science 2013.5.14(Tue)

#include <SPI.h>

// Arduinoボードの電源電圧がキットの電源電圧と同じ場合、
// 以下のマクロを有効にしてモジュールを直挿しできます。
// If working voltage of your arduino board is same as the kit, 
// you can enable macro code next two lines
// to put the kit on your arduino directly.

#define VCC  8
#define GND  9
#define SLAVE 10

void setup() {
#ifdef GND
  pinMode(GND, OUTPUT);
  digitalWrite(GND, LOW); 
#endif
#ifdef VCC
  pinMode(VCC, OUTPUT);
  digitalWrite(VCC, HIGH);
#endif
  pinMode(SLAVE, OUTPUT);
  digitalWrite(SLAVE, HIGH);

  Serial.begin(9600);
  SPI.begin();
  //SPI.setBitOrder(MSBFIRST);
  //SPI.setClockDivider(SPI_CLOCK_DIV4);
  //SPI.setDataMode(SPI_MODE0);
  
  Serial.flush();
  Serial.println();
  Serial.println();
  Serial.println("<*start*>");
}
/*******************************************************************************
*  ctime(tm,buff)                                                              *
*    millis関数の時間を日/時間:分:秒の文字列に変換する処理                     *
*                                                                              *
*     tm   : millis関数から返された時間を指定する                              *
*     buff : 日/時間:分:秒に変換したデータを格納するバッファ                   *
*******************************************************************************/
void ctime(unsigned long tm,char *buff)
{
     unsigned long time ;
     unsigned long day , h , m , s ;
     char c[6] ;
     
     day = tm / 86400000L ;        // 日(0-5)
     h = tm / 3600000L ;           // 時間(0-120)
     time = tm % 3600000L ;
     if (time != 0L) {
          m = time / 60000L ;      // 分(0-59)
          time = time % 60000L ;
          if (time != 0L) {
               s = time / 1000L ;  // 秒(0-59)
          } else s = 0 ;
     } else m = s = 0 ;
     // 文字列に変換
     sprintf(buff,"%02d:",h) ;
     sprintf(c,"%02d:",m) ;
     strcat(buff,c) ;
     sprintf(c,"%02d",s) ;
     strcat(buff,c) ;
}
void loop() {
  unsigned int thermocouple; // 14-Bit Thermocouple Temperature Data + 2-Bit
  unsigned int internal; // 12-Bit Internal Temperature Data + 4-Bit
  int disp; // display value 
  float disp_float;
  
  unsigned long tm;
  char buff[12];
  tm = millis();
  ctime(tm,(char *)buff);

  digitalWrite(SLAVE, LOW);                              //  Enable the chip
  thermocouple  = (unsigned int)SPI.transfer(0x00) << 8; //  Read high byte thermocouple
  thermocouple |= (unsigned int)SPI.transfer(0x00);      //  Read low byte thermocouple 
  internal      = (unsigned int)SPI.transfer(0x00) << 8; //  Read high byte internal
  internal     |= (unsigned int)SPI.transfer(0x00);      //  Read low byte internal 
  digitalWrite(SLAVE, HIGH);                             //  Disable the chip

  if((thermocouple & 0x0001) != 0) {
    Serial.print("ERROR: ");
    if ((internal & 0x0004) !=0) {
      Serial.print("SHORT_TO_VCC ");
    }
    if ((internal & 0x0002) !=0) {
      Serial.print("SHORT_TO_GND ");
    }
    if ((internal & 0x0001) !=0) {
      Serial.print("OPEN_CIRCUIT ");
    }    
    Serial.println();
  } else {
    if((thermocouple & 0x8000) == 0){ // 0℃以上   above 0 Degrees Celsius 
      disp = (thermocouple >> 2) * 25;
    } else {                          // 0℃未満   below zero
      disp = (0x3fff - (thermocouple >> 2) + 1)  * -25;
    }
    disp_float = ((float)disp) / 100;
    Serial.print(buff);
    Serial.print("\t");
    Serial.print(disp_float);
    Serial.println();
  }
  
  delay(999);
}
